@@ -22,6 +22,7 @@ module Data.CaseInsensitive.Internal ( CI
                                      , original
                                      , foldedCase
                                      , map
+                                     , traverse
                                      , FoldCase(foldCase)
                                      ) where
 
@@ -30,19 +31,21 @@ module Data.CaseInsensitive.Internal ( CI
 --------------------------------------------------------------------------------
 
 -- from base:
-import Data.Bool     ( (||) )
-import Data.Char     ( Char, toLower )
-import Data.Eq       ( Eq, (==) )
-import Data.Function ( on )
-import Data.Monoid   ( Monoid, mempty, mappend )
-import Data.Ord      ( Ord, compare )
-import Data.String   ( IsString, fromString )
-import Data.Data     ( Data )
-import Data.Typeable ( Typeable )
-import Data.Word     ( Word8 )
-import Prelude       ( (.), fmap, (&&), (+), (<=), otherwise )
-import Text.Read     ( Read, readPrec )
-import Text.Show     ( Show, showsPrec )
+import Control.Applicative (Applicative)
+import Data.Bool      ( (||) )
+import Data.Char      ( Char, toLower )
+import Data.Eq        ( Eq, (==) )
+import Data.Function  ( on )
+import Data.Monoid    ( Monoid, mempty, mappend )
+import Data.Ord       ( Ord, compare )
+import Data.String    ( IsString, fromString )
+import Data.Data      ( Data )
+import Data.Typeable  ( Typeable )
+import Data.Word      ( Word8 )
+import Prelude        ( (.), fmap, (&&), (+), (<=), otherwise )
+import Text.Read      ( Read, readPrec )
+import Text.Show      ( Show, showsPrec )
+import Data.Semigroup ( Semigroup, (<>) )
 
 import qualified Data.List as L ( map )
 
@@ -106,9 +109,16 @@ unsafeMk s = CI s s
 map :: FoldCase s2 => (s1 -> s2) -> (CI s1 -> CI s2)
 map f = mk . f . original
 
+-- | Transform the original string-like value but keep it case insensitive.
+traverse :: (FoldCase s2, Applicative f) => (s1 -> f s2) -> CI s1 -> f (CI s2)
+traverse f = fmap mk . f . original
+
 instance (IsString s, FoldCase s) => IsString (CI s) where
     fromString = mk . fromString
 
+instance Semigroup s => Semigroup (CI s) where
+    CI o1 l1 <> CI o2 l2 = CI (o1 <> o2) (l1 <> l2)
+
 instance Monoid s => Monoid (CI s) where
     mempty = CI mempty mempty
     CI o1 l1 `mappend` CI o2 l2 = CI (o1 `mappend` o2) (l1 `mappend` l2)
